-- EquipNSpecReminder Addons --
-- Create the addons in memory
local EquipNSpecReminder = LibStub("AceAddon-3.0"):NewAddon("EquipNSpecReminder", "AceEvent-3.0", "AceHook-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("EquipNSpecReminder");

local IsEquipChanged = false;
local specAutoChanged = false;
local equipAutoChanged = false;

local OPTION_SELECT_WINDOW = "-- "..L["Show selection window"].." --";
local OPTION_REMIND_ONLY   = "-- "..L["Only remind"].." --";
local OPTION_DO_NOTHING    = "-- "..L["Do nothing"].." --";
local ANNOUNCE_RAID = L["Raid Warning"];
local ANNOUNCE_CHAT = L["Chat Window"];
local ANNOUNCE_NO   = "-- "..L["Do not announce"].." --";
local SPEC_IDENTIFIER = "spec"


local myOptionsTable = {
   type = "group",
   name = "Equip'n'Spec Reminder",
   get = function(item) return EquipNSpecReminder.db.profile[item[#item]] end,
   set = function(item, value) EquipNSpecReminder.db.profile[item[#item]] = value end,
   args = {
      headerEquip = {
         type = "header",
         name = L["Equipment action performed when spec is changed"],
         order = 1,
      },
      defaultEquipFirst = {
         type = "select",
         name = L["Equipment set used when changing to primary spec"],
         style = "dropdown",
         values = function()
            local EquipSets = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_REMIND_ONLY] = OPTION_REMIND_ONLY, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}
            for i=1,GetNumEquipmentSets() do
               local name,_,_ = GetEquipmentSetInfo(i)
               EquipSets[name] = name
            end
            return EquipSets
         end,
         order = 2,
         width = "double",
      },
      defaultEquipSec = {
         type = "select",
         name = L["Equipment set used when changing to secondary spec"],
         style = "dropdown",
         values = function()
            local EquipSets = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_REMIND_ONLY] = OPTION_REMIND_ONLY, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}
            for i=1,GetNumEquipmentSets() do
               local name,_,_ = GetEquipmentSetInfo(i)
               EquipSets[name] = name
            end
            return EquipSets
         end,
         order = 3,
         width = "double"
      },
      blankLineOne  = {
         type = "description",
         name = " ",
         fontSize = "large",
         width = "full",
         order = 4,
      },
      headerSpec = {
         type = "header",
         name = L["Spec action performed when equipment set is changed"],
         order = 5,
      },
      defaultSpec = {
        type = "group",
        inline = true,
        name = "",
        desc = "",
        args = {
        },
      },
      blankLineTwo  = {
         type = "description",
         name = " ",
         fontSize = "large",
         width = "full",
         order = 100,
      },
      headerBg = {
         type = "header",
         name = L["Actions performed when entering battleground"],
         order = 200,
      },
      defaultEquipBg = {
         type = "select",
         name = L["Equipment set used when entering battleground"],
         style = "dropdown",
         values = function()
            local EquipSets = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_REMIND_ONLY] = OPTION_REMIND_ONLY, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}
            for i=1,GetNumEquipmentSets() do
               local name,_,_ = GetEquipmentSetInfo(i)
               EquipSets[name] = name
            end
            return EquipSets
         end,
         order = 201,
         width = "double",
      },
      defaultSpecBg = {
         type = "select",
         name = L["Spec set when entering battleground"],
         style = "dropdown",
         values = function()
            local Specs = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_REMIND_ONLY] = OPTION_REMIND_ONLY, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}

            for i=1,GetNumSpecGroups() do
               local spec = GetSpecialization(false, false, i)
               local specName = spec and select(2, GetSpecializationInfo(spec)) or "None"
               Specs[SPEC_IDENTIFIER..i] = specName.." ("..spec..")"; --the text of the menu item
            end

            return Specs
         end,
         order = 202,
         width = "double",
      },
      blankLineThree  = {
         type = "description",
         name = " ",
         fontSize = "large",
         width = "full",
         order = 203,
      },
      headerAnnounce = {
         type = "header",
         name = L["Notification of automatically performed changes"],
         order = 300,
      },
      announce = {
         type = "select",
         name = L["Announce change in"],
         style = "dropdown",
         values = {[ANNOUNCE_NO] = ANNOUNCE_NO, [ANNOUNCE_RAID] = ANNOUNCE_RAID, [ANNOUNCE_CHAT] = ANNOUNCE_CHAT},
         order = 301,
      },
   }
};


local defaults = {
   profile = {
      defaultEquipBg    = OPTION_SELECT_WINDOW,
      defaultSpecBg     = OPTION_SELECT_WINDOW,
      defaultEquipFirst = OPTION_SELECT_WINDOW,
      defaultEquipSec   = OPTION_SELECT_WINDOW,
      defaultSpec       = OPTION_SELECT_WINDOW,
      announce          = ANNOUNCE_RAID,
   },
}


-------------------------------------------------------------------------------
-- Hooks and boostrap code
-------------------------------------------------------------------------------

-- We're alive!
function EquipNSpecReminder:OnInitialize()

   self.db = LibStub("AceDB-3.0"):New("EquipNSpecReminderDB", defaults);

   LibStub("AceConfig-3.0"):RegisterOptionsTable("EquipNSpecReminder", myOptionsTable, nil);
   local generalOptions = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EquipNSpecReminder", "Equip'n'Spec Reminder");
   generalOptions.default = function() self:SetDefaultOptions() end;
end


function EquipNSpecReminder:OnEnable()
   -- Do more initialization here, that really enables the use of your addon.
   -- Register Events, Hook functions, Create Frames, Get information from
   -- the game that wasn't available in OnInitialize

   self:RegisterEvent("PLAYER_ENTERING_WORLD");
   self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
   self:RegisterEvent("EQUIPMENT_SWAP_FINISHED");
   self:RegisterEvent("EQUIPMENT_SETS_CHANGED");
   self:RegisterEvent("CHARACTER_POINTS_CHANGED");
   self:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND");
   self:RegisterEvent("ZONE_CHANGED_NEW_AREA");

   self:HookScript(UIParent, "OnUpdate", "OnMyUpdate")

   if theFrame == nil then
      theFrame = CreateFrame("frame","MyAddonFrame")
      theFrame:SetBackdrop({
      bgFile="Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
      edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
      tile=1, tileSize=32, edgeSize=14,
      insets={left=4, right=4, top=4, bottom=4}
      })
      theFrame:SetWidth(300)
      theFrame:SetHeight(110)
      theFrame:SetPoint("CENTER",UIParent)
      theFrame:EnableMouse(true)
      theFrame:SetMovable(true)
      theFrame:RegisterForDrag("LeftButton")

      theFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
      theFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
      theFrame:SetFrameStrata("FULLSCREEN_DIALOG")

      theFrame.Headline=theFrame:CreateFontString(nil,"OVERLAY","GameTooltipText");
      theFrame.Headline:SetPoint("TOP",theFrame,"TOP",0,-10);
      theFrame.Headline:SetHeight(10);
      theFrame.Headline:SetWidth(300);
      theFrame.Headline:SetJustifyH("CENTER");
      theFrame.Headline:SetTextColor(0.12,0.56,1);
      theFrame.Headline:SetText("--- Equip'n'Spec Reminder ---");

      theFrame.MessageText=theFrame:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall");
      theFrame.MessageText:SetPoint("TOP",theFrame.Headline,"TOP",0,-20);
      theFrame.MessageText:SetHeight(10);
      theFrame.MessageText:SetWidth(300);
      theFrame.MessageText:SetJustifyH("CENTER");
      --theFrame.MessageText:SetTextColor(1,0,0);

      theFrame.EquipData = CreateFrame("Frame", "EquipDropDown", theFrame, "UIDropDownMenuTemplate");
      theFrame.EquipData:SetPoint("TOP",theFrame.MessageText,"TOP",0,-20);
      UIDropDownMenu_SetWidth(theFrame.EquipData,130);
      UIDropDownMenu_SetButtonWidth(theFrame.EquipData, 154);
      UIDropDownMenu_JustifyText(theFrame.EquipData, "LEFT");
      UIDropDownMenu_Initialize(theFrame.EquipData, DropdownData_Initialise);

      theFrame.OkButton = CreateFrame("button","MyAddonButtonOk", theFrame, "EquipNSpecButtonTemplate")
      theFrame.OkButton:SetPoint("BOTTOM", theFrame, "BOTTOM", -60, 10)
      theFrame.OkButton:SetText(OKAY)
      theFrame.OkButton:SetScript("OnClick", function(self) AcceptChange() end)

      theFrame.CancelButton = CreateFrame("button","MyAddonButtonCancel", theFrame, "EquipNSpecButtonTemplate")
      theFrame.CancelButton:SetPoint("BOTTOM", theFrame, "BOTTOM", 60, 10)
      theFrame.CancelButton:SetText(CANCEL)
      theFrame.CancelButton:SetScript("OnClick", function(self) self:GetParent():Hide() end)

      theFrame:Hide()
   end
end


function EquipNSpecReminder:OnDisable()
   -- Unhook, Unregister Events, Hide frames that you created.
   -- You would probably only use an OnDisable if you want to
   -- build a "standby" mode, or be able to toggle modules on/off.
end


function EquipNSpecReminder:UpdateConfigMenu()

   myOptionsTable.args.defaultSpec.args = {};

   if GetNumEquipmentSets() ~= 0 then
      local Specs = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_REMIND_ONLY] = OPTION_REMIND_ONLY, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}
      for i=1,GetNumEquipmentSets() do
         local name,_,_ = GetEquipmentSetInfo(i)
         myOptionsTable.args.defaultSpec.args[name] = {
            type = "select",
            name = L["Spec set when changing to equipment set"].." \'"..name.."\'",
            style = "dropdown",
            values = function()
               for i=1, GetNumSpecGroups() do
                  local spec = GetSpecialization(false, false, i)
                  local specName = spec and select(2, GetSpecializationInfo(spec)) or "None"
                  Specs[SPEC_IDENTIFIER..i] = specName.." ("..spec..")"; --the text of the menu item
               end
               return Specs;
            end,
            width = "double",
            order = (20 + i),
         }
      end
   end

   -- remove entries from DB where the equip set is removed
   for key,v in pairs(self.db.profile) do
      if myOptionsTable.args.defaultSpec.args[key] == nil and myOptionsTable.args[key] == nil then
         self.db.profile[key] = nil;
      end
   end

   -- set default value for new items
   for key,v in pairs(myOptionsTable.args.defaultSpec.args) do
      if self.db.profile[key] == nil then
         self.db.profile[key] = defaults.profile.defaultSpec;
      end
   end
end


-------------------------------------------------------------------------------
-- Event handler functions
-------------------------------------------------------------------------------
function EquipNSpecReminder:PLAYER_ENTERING_WORLD()
   -- Calling UpdateConfigMenu does not work in the OnInitialize routine as
   -- player information might not be completely available at that point of time.
   self:UpdateConfigMenu()
end


function EquipNSpecReminder:EQUIPMENT_SETS_CHANGED()
   self:UpdateConfigMenu()
end

function EquipNSpecReminder:CHARACTER_POINTS_CHANGED()
   self:UpdateConfigMenu()
end

function EquipNSpecReminder:EQUIPMENT_SWAP_FINISHED(Aasdf,success, equipSetName)

   if equipAutoChanged == false and success == true then
      -- check what shall be done when changing to the new equipment set
      if self.db.profile[equipSetName] ~= nil then
         if self.db.profile[equipSetName] == OPTION_DO_NOTHING then
            -- do nothing
         elseif self.db.profile[equipSetName] == OPTION_SELECT_WINDOW then
            -- show selection frame
            self:ShowSpecSelectFrame();
         elseif self.db.profile[equipSetName] == OPTION_REMIND_ONLY then
            message(L["Equipment was changed, please remember to also change spec."]);
         else
            local specId = tonumber(strsub(self.db.profile[equipSetName], -1));
            -- check if spec is different to currently used spec
            if GetActiveSpecGroup() ~= specId then
               -- check if spec exists
               if GetNumSpecGroups() >= specId then
                  specAutoChanged = true;
                  SetActiveSpecGroup(specId);
                  local spec = GetSpecialization(false, false, specId)
                  local specName = spec and select(2, GetSpecializationInfo(spec)) or "None"
                  self:AnnounceChange (L["spec"], specId.." ("..specName..")")
               else
                  -- Spec does not exist. Set to default.
                  self.db.profile[equipSetName] = defaults.profile.defaultSpec;
                  -- show selection frame
                  self:ShowSpecSelectFrame();
               end
            end
         end
      end
   else
      -- TODO: only announce confirmation when automatic change has really happened
      -- RaidNotice_AddMessage(RaidWarningFrame, "Automatically changed to equipment set \"".. sdfgdfgdfg.."\"", ChatTypeInfo["RAID_WARNING"],5)
   end
   equipAutoChanged = false;
end


function EquipNSpecReminder:ACTIVE_TALENT_GROUP_CHANGED()

   if specAutoChanged == false then
      -- check if there was a change to the primary spec and if a
      -- default equipment is defined
      if GetActiveSpecGroup()==1 then
         if self.db.profile.defaultEquipFirst == OPTION_DO_NOTHING then
            -- do nothing
         elseif self.db.profile.defaultEquipFirst == OPTION_SELECT_WINDOW then
            -- show selection frame
            self:ShowEquipSelectFrame();
         elseif self.db.profile.defaultEquipFirst == OPTION_REMIND_ONLY then
            message(L["Spec was changed, please remember to also change equipment."]);
         else
            -- check if equipment set exists
            icon, lessIndex = GetEquipmentSetInfoByName(self.db.profile.defaultEquipFirst)
            if icon ~= nil then
               equipAutoChanged = true;
               UseEquipmentSet(self.db.profile.defaultEquipFirst)
               self:AnnounceChange (L["equipment set"], self.db.profile.defaultEquipFirst)
            else
               -- Equipment set does not exist. Set to default.
               self.db.profile.defaultEquipFirst = defaults.profile.defaultEquipFirst;
               -- show selection frame
               self:ShowEquipSelectFrame();
            end
         end

      -- check if there was a change to the secondary spec and if a
      -- default equipment is defined
      elseif GetActiveSpecGroup()==2 then
         if self.db.profile.defaultEquipSec == OPTION_DO_NOTHING then
            -- do nothing
         elseif self.db.profile.defaultEquipSec == OPTION_SELECT_WINDOW then
            -- show selection frame
            self:ShowEquipSelectFrame();
         elseif self.db.profile.defaultEquipSec == OPTION_REMIND_ONLY then
            message(L["Spec was changed, please remember to also change equipment."]);
         else
            -- check if equipment set exists
            icon, lessIndex = GetEquipmentSetInfoByName(self.db.profile.defaultEquipSec)
            if icon ~= nil then
               equipAutoChanged = true;
               UseEquipmentSet(self.db.profile.defaultEquipSec)
               self:AnnounceChange (L["equipment set"], self.db.profile.defaultEquipSec)
            else
               -- Equipment set does not exist. Set to default.
               self.db.profile.defaultEquipSec = defaults.profile.defaultEquipSec;
               -- show selection frame
               self:ShowEquipSelectFrame();
            end
         end
      end
   end
   specAutoChanged = false;
end



function EquipNSpecReminder:ZONE_CHANGED_NEW_AREA()
   --print "ZONE_CHANGED_NEW_AREA";
end

function EquipNSpecReminder:PLAYER_ENTERING_BATTLEGROUND()
   -- handle equip when entering battleground
   if self.db.profile.defaultEquipBg == OPTION_DO_NOTHING then
      -- do nothing
   elseif self.db.profile.defaultEquipBg == OPTION_SELECT_WINDOW then
      -- show selection frame
      self:ShowEquipSelectFrame();
   elseif self.db.profile.defaultEquipBg == OPTION_REMIND_ONLY then
      message(L["Entered battleground. Please remember to change equipment."]);
   else
      -- check if equipment set exists
      icon, lessIndex = GetEquipmentSetInfoByName(self.db.profile.defaultEquipBg)
      if icon ~= nil then
         equipAutoChanged = true;
         UseEquipmentSet(self.db.profile.defaultEquipBg)
         self:AnnounceChange (L["equipment set"], self.db.profile.defaultEquipBg)
      else
         -- Equipment set does not exist. Set to default.
         self.db.profile.defaultEquipBg = defaults.profile.defaultEquipBg;
         -- show selection frame
         self:ShowEquipSelectFrame();
      end
   end

   -- handle spec when entering battleground
   if self.db.profile.defaultSpecBg == OPTION_DO_NOTHING then
      -- do nothing
   elseif self.db.profile.defaultSpecBg == OPTION_SELECT_WINDOW then
      -- show selection frame
      self:ShowSpecSelectFrame();
   elseif self.db.profile.defaultSpecBg == OPTION_REMIND_ONLY then
      message(L["Entered battleground. Please remember to change spec."]);
   else
      local specId = tonumber(strsub(self.db.profile.defaultSpecBg, -1));
      -- check if spec is different to currently used spec
      if GetActiveSpecGroup() ~= specId then
         -- check if spec exists
         if GetNumSpecGroups() >= specId then
            specAutoChanged = true;
            SetActiveSpecGroup(specId);
            local spec = GetSpecialization(false, false, specId)
            local specName = spec and select(2, GetSpecializationInfo(spec)) or "None"
            self:AnnounceChange (L["spec"], specId.." ("..pecName..")")
         else
            -- Spec does not exist. Set to default.
            self.db.profile.defaultSpecBg = defaults.profile.defaultSpecBg;
            -- show selection frame
            self:ShowSpecSelectFrame();
         end
      end
   end

end



-------------------------------------------------------------------------------
-- User interface
-------------------------------------------------------------------------------

-- Timeout function
function EquipNSpecReminder:OnMyUpdate(arg1, time) -- "time" is in secs so may me small like 0.020s
   if equipAutoChanged == true then
      timeElapsed = timeElapsed + time;

      if(timeElapsed >= 3) then
         timeElapsed = 0;
         equipAutoChanged = false;
         RaidNotice_AddMessage(RaidWarningFrame, "Automatic Equipment change could not be performed!", ChatTypeInfo["RAID_WARNING"],10)
      end
   else
      timeElapsed = 0;
   end
end

-- set all configuration values to default
function EquipNSpecReminder:SetDefaultOptions()
   local profileDB = self.db.profile;

   -- set default values for static options
   profileDB.defaultEquipFirst  = defaults.profile.defaultEquipFirst;
   profileDB.defaultEquipSec    = defaults.profile.defaultEquipSec;
   profileDB.defaultEquipBg     = defaults.profile.defaultEquipBg;
   profileDB.defaultSpecBg      = defaults.profile.defaultSpecBg;
   profileDB.announce           = defaults.profile.announce;
   -- set default value for dynamically created items
   for key,v in pairs(myOptionsTable.args.defaultSpec.args) do
      self.db.profile[key] = defaults.profile.defaultSpec;
   end

   --force config dialog to show new state of options
   LibStub("AceConfigRegistry-3.0"):NotifyChange("EquipNSpecReminder");
end

-- Announce changes
function EquipNSpecReminder:AnnounceChange (what, toWhat)
   local announcmentText = "Automatically changed to "..what.." \"".. toWhat .."\"";
   if self.db.profile.announce == ANNOUNCE_RAID then
      RaidNotice_AddMessage(RaidWarningFrame, announcmentText, ChatTypeInfo["RAID_WARNING"],5)
   elseif self.db.profile.announce == ANNOUNCE_CHAT then
      print (announcmentText);
   else
      -- do nothing
   end
end


-------------------------------------------------------------------------------
-- Functions for the selection frame (selection of equip or spec)
-------------------------------------------------------------------------------

-- show frame to select an equipment set
function EquipNSpecReminder:ShowEquipSelectFrame ()
   -- show selection frame
   if GetNumEquipmentSets() ~= 0 then
      IsEquipChanged = false;
      theFrame.MessageText:SetText(L["Please also change equipment. Choose set:"]);
      local name,_,_ = GetEquipmentSetInfo(1);
      UIDropDownMenu_SetSelectedValue(theFrame.EquipData, name);
      _G[theFrame.EquipData:GetName().."Text"]:SetText(name);
      theFrame:Show()
   else
      -- no equipment set to select from
      message(L["-- Equip'n'Spec Reminder --\n\nNo chosable equipment sets available."]);
   end
end


-- show frame to select spec
function EquipNSpecReminder:ShowSpecSelectFrame ()
   -- show selection frame
   IsEquipChanged = true;
   theFrame.MessageText:SetText(L["Please also change your spec. Choose spec:"]);
   if GetActiveSpecGroup()==1 and GetNumSpecGroups() > 1 then
      UIDropDownMenu_SetSelectedValue(theFrame.EquipData, 2);
   else
      UIDropDownMenu_SetSelectedValue(theFrame.EquipData, 1);
   end
--   _G[theFrame.EquipData:GetName().."Text"]:SetText(name);
   theFrame:Show()
end


-- OnClick event for dropdown select item
function DropdownData_OnClick(item)
   UIDropDownMenu_SetSelectedValue(theFrame.EquipData, item.value);
end

--
function AcceptChange()
   if IsEquipChanged then
      specAutoChanged = true;
      SetActiveSpecGroup(UIDropDownMenu_GetSelectedValue(theFrame.EquipData));
   else
      equipAutoChanged = true;
      UseEquipmentSet(UIDropDownMenu_GetSelectedValue(theFrame.EquipData));
   end
   theFrame:Hide()
end


function DropdownData_Initialise()
   if IsEquipChanged then
      local info = UIDropDownMenu_CreateInfo();

      for i=1,GetNumSpecGroups() do
         local spec = GetSpecialization(false, false, i)
         local specName = spec and select(2, GetSpecializationInfo(spec)) or "None"

         info.text = specName.." ("..spec..")"; --the text of the menu item
         info.func = DropdownData_OnClick; --sets the function to execute when this item is clicked
         info.value = i; -- the value of the menu item. This can be a string also.
         info.checked = nil; --initially set the menu item to being unchecked with a yellow tick
         UIDropDownMenu_AddButton(info, level); --Adds the new button to the drop down menu specified in the UIDropDownMenu_Initialise function. In this case, it's MyDropDownMenu
      end
   else
      if GetNumEquipmentSets() ~= 0 then
         local info = UIDropDownMenu_CreateInfo();

         for i=1,GetNumEquipmentSets() do
            local name,_,_ = GetEquipmentSetInfo(i)
            info.text = name; --the text of the menu item
            info.func = DropdownData_OnClick; --sets the function to execute when this item is clicked
            info.value = name; -- the value of the menu item. This can be a string also.
            info.checked = nil; --initially set the menu item to being unchecked with a yellow tick
            UIDropDownMenu_AddButton(info, level); --Adds the new button to the drop down menu specified in the UIDropDownMenu_Initialise function. In this case, it's MyDropDownMenu
         end
      end
   end
end
